#!/usr/bin/python
#! -*- coding: utf-8 -*-

''' Firefox Widgets Installer
    original widgets are done by Osmo Salomaa <otsaloma@cc.hut.fi>
    installs the new-style widgets by means of fastsheep's script:
    [http://ubuntuforums.org/showthread.php?t=369596]
    
    gui-author: Paul Eipper
    date: 03-Jun-2007
    license: GPL.v2 or latest
'''

import os
import gtk
import gettext
import kiwi.ui.dialogs as dialog
from kiwi.ui.delegates import Delegate
from subprocess import Popen, PIPE

path_file="paths"
max_saved_paths=5
default_install_path="/usr/lib/firefox"
default_paths = [ default_install_path, "/usr/local/firefox32", "/usr/local/swiftfox", "/usr/lib/swiftfox", "/usr/local/Iceweasel32" ]


class FFWidgetsInterface(Delegate):
    ''' interface class, to handle buttons and such '''
    gladefile = 'ff_widgets'

    def __init__(self):
       ''' initialize with default path '''
       Delegate.__init__(self)
       self.toplevel.connect('delete-event', self.quit_if_last)
             
       if os.path.exists(path_file):
           previous_paths = open(path_file, "r").read().splitlines()
           
           # Remove duplicate entries 
           for i in previous_paths:
               while previous_paths.count(i) > 1:
                   previous_paths.remove(i)
           
           # Strip leading and trailing whitespace
           for i in range(len(previous_paths)):
               previous_paths[i] = previous_paths[i].strip()
           
           # take the last max_saved_paths number of entries for display
           previous_paths = previous_paths[-max_saved_paths:]  
           if len(previous_paths[-1]) != 0:
               self.combo_entry1.set_text(previous_paths[-1])         
           for i in range(len(previous_paths)):
               #previous_paths[i] = previous_paths[i].replace("\n", "")
               if previous_paths[i] not in default_paths and previous_paths[i] != "":
			       self.combo_box1.append_text(previous_paths[i])
           

       # If there is no path in the entry, set it to the default installation path
       if len(self.combo_entry1.get_text()) == 0:
           self.combo_entry1.set_text(default_install_path)
       
       # Append a list of installation paths
       for i in default_paths:
           self.combo_box1.append_text(i)

    def on_gnomedruid__cancel(self, *args):
        ''' quit application '''
        self.hide_and_quit()
        
    def on_last_page__finish(self, *args):
        ''' apply selected actions '''
        path = self.combo_entry1.get_text()
        
        # log what path we installed to
        open(path_file, "a").write(path + "\n")
               
         
        do = FFWidgets(path)
        if self.radiobutton_install.get_active():
            output = do.install()
            change_type="Installation"
        elif self.radiobutton_remove.get_active():
            output = do.remove()
            change_type="Removal"

	dialog.info('%s complete.  Please Restart Firefox.' % change_type, output)
        self.hide_and_quit()
        
    def on_button_browse__clicked(self, *args):
        ''' browse for firefox folder '''
        folder = self.open_folder(folder=self.combo_entry1.get_text())
        if folder:
            self.combo_entry1.set_text(folder)
        
    def open_folder(self, title='', parent=None, folder=None):
        ''' selects a folder, based on kiwi.ui.dialogs.open '''
        _ = dialog._
        
        filechooser = gtk.FileChooserDialog(title or _('Open'), parent,
                                            gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                             gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        if folder:
            filechooser.set_current_folder(folder)

        response = filechooser.run()
        if response != gtk.RESPONSE_OK:
            filechooser.destroy()
            return

        path = filechooser.get_filename()
        if path and os.access(path, os.R_OK):
            filechooser.destroy()
            return path
        
        abspath = os.path.abspath(path)

        dialog.error(_('Could not open file "%s"') % abspath,
                     _('The file "%s" could not be opened. '
                       'Permission denied.') %  abspath)
        path = None
        
        filechooser.destroy()
        return path
        
    def run(self):
        ''' open window and run gtk main loop '''
        self.show()
        gtk.main()


class FFWidgets(object):
    ''' handles applying the changes and doing the stuff we want '''
    def __init__(self, path):
        self.admin = "gksudo"
        self.cmd = "./install "
        self.path = path.strip()
        
    def install(self):
        print 'Installing on :', self.path
        return self.execute('-i')
        
    def remove(self):
        print 'Removing from :', self.path
        return self.execute('-r')
    
    def execute(self, action=''):
        assert action
        args = "-p=%s %s --gui" % (self.path, action)
        args = self.cmd + args
        output = Popen([self.admin, args], stdin=PIPE, stdout=PIPE, close_fds=True)
        return output.stdout.read()
        

app = FFWidgetsInterface()
app.run()
